#define	_POSIX_SOURCE	1
/* 
 * decoding and exec module for twd.
 * dannyb@bga.com	10/27/96
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>

#include "twoway.h"		/* only for struct x10msg declaration */
#include "rcfile.h"
#include "exec.h"

#define TRUE	1
#define FALSE	0


int x10func(struct x10msg *s, char ***rcfile) {
  int line, word=0;
  int unit = -1;
  int primecode = -1;
  char **cmdline;

  /* compare each line in rcfile against calling args */
  for(line = 0; rcfile[line] != NULL; line++) {
	/* look for "event" keyword */
	word = 0;
	if (!samestring(rcfile[line][word], "event"))
		continue;

	/* compare housecodes */
	word++;
	if (rcfile[line][word] == NULL)
		continue;
	if ( (*rcfile[line][word] != '*') &&
	     (toupper(*rcfile[line][word]) != toupper(s->house)) )
		continue;


	/* compare unitcode */
	word++;
	if (rcfile[line][word] == NULL)
		continue;
	if (*rcfile[line][word] != '*') {
		unit = atoi(rcfile[line][word]);
		if ( (unit < 1) || (unit > 16) )
			continue;	/* field out of range */
		primecode = (1 << (unit-1));
		if ( (s->modules & primecode) == 0 )
			continue;	/* units don't match */
	}

	/* compare function code */
	word++;
	if (rcfile[line][word] == NULL)
		continue;
	if (*rcfile[line][word] != '*') {
		if (!samestring(rcfile[line][word], s->func))
			continue;
	}

	word++;
	if (rcfile[line][word] == NULL)
		continue;

	/* We have a match if we have gotten this far */

	/* Substitute variables */
	cmdline = subargs(&rcfile[line][word], s);
	if (cmdline == NULL)
		return -1;

	(void)fastsys(cmdline);
	free2array(cmdline);
  }
  return 0;
} 

char **subargs(char **array, struct x10msg *s) {
/*
 * Fill in any variables from the rcfile into a copy of the supplied array 
 * and return the copy. The result must be free()ed when no longer needed.
*/ 
  int word;
  static char **cmdline;

  /* copy the array so we don't alter the original */
  cmdline = dup2array(array);
  if(cmdline == NULL)
	return cmdline;

  /* replace any variables with calling parameters */
  for(word=0; (cmdline[word] != NULL); word++) {
	if (samestring(cmdline[word], "$HOUSE")) {
		sprintf(cmdline[word], "%c", s->house);
	}
	if (samestring(cmdline[word], "$MODULES")) {
		sprintf(cmdline[word], "%05i", s->modules);
	}
	if (samestring(cmdline[word], "$FUNCTION")) {
		sprintf(cmdline[word], "%2s", s->func);
	}
  }

  return cmdline;
}


int fastsys(char **args) {
  int exitstatus;
  pid_t kidpid;
  char *array[4];
  static int devnull = -1;

  array[0] = "sh";
  array[1] = "-c";
  array[2] = array2tos(args, ' ');
  array[3] = NULL;

  if (array[2] == NULL)
	return -1;

  if (devnull == -1)
	devnull = open("/dev/null", O_RDWR);

  kidpid = fork();
  switch((int)kidpid) {
	case -1:
		perror("fastsys: fork()");
		return -1;
	case 0:	/* child code */
		kidpid = fork();
		switch((int)kidpid) {
			case -1:	/* error */
				exit(1);
			case 0:		/* grandchild */
				if (devnull != -1) {
					close(STDIN_FILENO);
					close(STDOUT_FILENO);
					close(STDERR_FILENO);
					dup(devnull);
					dup(devnull);
					dup(devnull);
				}
				execvp("/bin/sh", array);
				/* exec call must have failed */
				exit(1);
			default:	/* child orfans gradchild */
				exit(0);
		}
	default:
		if (waitpid(kidpid, &exitstatus, 0) == -1) {
			perror("fastsys: waitpid()");
			return -1;
		} else {
			return exitstatus;
		}
  }
}
