#define _POSIX_SOURCE   1
/*
 * sendx.c
 * Sample client app for 'twd' server
 * dannyb@bga.com 10/30/96
 *
 * Copyright (c) 1996 Daniel R. Barrett.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
*/
#include <stdio.h>
#include <getopt.h>
#include <signal.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "client.h"

#define TRUE	1
#define FALSE	0

/* prototypes */
void usage(char *progname);
int main(int argc, char **argv);

int main(int argc, char **argv) {
  int unit;
  int i;
  struct x10msg m;

  if (argc < 3) {
	usage(argv[0]);
	exit(1);
  }

  /* first arg is house code */
  m.house = toupper(*argv[1]);
  if ( (m.house < 'A') || (m.house > 'P') ) {
	usage(argv[0]);
	exit(1);
  }

  /* next args are unit codes until one is not an int */
  m.modules = 0;
  for(i=2; i<argc; i++) {
	unit = 0;
	unit = atoi(argv[i]);
	if (unit == 0)	/* atoi failed */
		break;
	if ( (unit>0) && (unit<17) )
		m.modules |= 1 << (unit-1);
	else {		/* out of range */
		usage(argv[0]);
		exit(1);
	}
  }

  /* first non-numerical arg after housecode is a function code */
  if (i < argc)
	strncpy(m.func, argv[i], 2);
  m.func[2] = '\0';

  if (++i < argc)
	m.brdi = atoi(argv[i]);
  else
	m.brdi = 0;

  if (x10getqid() == -1) {
	perror("error connecting to server\n");
	exit(1);
  }

  if (x10send(&m) != 0) {
	perror("Error sending to server\n");
	exit(1);
  }

  exit(0);
}

void usage(char *progname) {
  printf("usage:\n %s [house] <module ...> [function] <DI/BR count>\n", 
progname);
  printf("  house:	House code. (A-P)\n");
  printf("  module:	Module code(1-16). multiple values allowed.\n");
  printf("  function:	ON, OF, BR, DI, A0, L1, HA, HR,\n");
  printf("		L0, EC, P0, P1, ED, S1, S0, SR\n");
  printf("  BR/DI:	repeat count for BR and DI commands(1-20).\n");
  return;
}
