#define	_POSIX_SOURCE	1
/* 
 * Server routines for twd.
 * dannyb@bga.com	10/27/96
*/
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include "server.h"

#define TRUE    	1
#define FALSE   	0

/* global variables */
int maxclients = 0;		/* max number of concurrent clients */
long *clientarray = NULL;	/* array of client process PIDs */


int init_clients(int count) {
  if (count < 1)
	return 0;
  maxclients = count;
  clientarray = (long *)calloc(count, sizeof(long));
  if (clientarray == NULL)
	return -1;
  else {
	int i;
	for(i=0; i<count; i++)		/* 0 = "open slot" indicator */
		clientarray[i] = 0;
  }

  return 0;
}


int get_input(long pid, long qid, struct x10msg *r) {
/*
 * Read and process the next message from the IPC queue.
*/

  if (peek_qid(qid, pid) != TRUE)
        return 0;  
  if (rec_msg(qid, pid, r) < 0)
        return -1;
  switch(r->func[0]) {
        case '+':	/* Client add request */
                (void)add_client(qid, r->from);
                break;
        case '-':	/* Client delete request */
                (void)del_client(qid, r->from);
                break;
        default:	/* Just an X10 command */
                return 1;
  }
  return 0;  
}




int add_client(long qid, long id) {
  struct x10msg m;
  int i;
  int success = FALSE;
  for(i=0; i<maxclients; i++) {		/* find open slot in array */
	if(clientarray[i] == 0) {
		clientarray[i] = id;
		success = TRUE;
		break;
	}
  }
  m.to = id;				/* Client's pid */
  m.from = (long)getpid();		/* Server's pid */
  if (success) {
	strcpy(m.func, "+");
	return send_msg(qid, &m);
  } else {
	strcpy(m.func, "-");
	return send_msg(qid, &m);
  }
}

int del_client(long qid, long id) {
  int i;
  struct x10msg r;  
  for(i=0; i<maxclients; i++) {
	if(clientarray[i] == id) {
		clientarray[i] = -1;
		while(peek_qid(qid, id) == TRUE) {
			(void)rec_msg(qid, id, &r);
		}
		return 0;
	}
  }
  return 0;	/* id is not in array */
}


int tell_clients(long qid, struct x10msg *m) {
  int i;
  for(i=0; i<maxclients; i++) {
	if(clientarray[i] != -1) {
		/* be sure this client is still alive */
		if (kill((pid_t)clientarray[i], 0) != 0) {
			(void)del_client(qid, clientarray[i]);
			continue;
		}
		m->to  = clientarray[i];
		m->from = (long)getpid();
		if (send_msg(qid, m) != 0)
			return -1;
	}
  }
  return 0;
}
