#define	_POSIX_SOURCE	1
/*
 * twd.c
 * IPC-based Linux server for Home Intelligence Corp. Two-Way(R) interface.
 * dannyb@bga.com 10/27/96
 *
 * Copyright (c) 1996 Daniel R. Barrett.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <string.h>

#include "server.h"
#include "rcfile.h"
#include "exec.h"
#include "twoway.h"

#define TRUE    	1
#define FALSE   	0

/* prototypes */
int main(int argc, char *argv[]);
void die(char *reason);
void cleanup(void);
void exit_handler(int signum);

/* global variables */
int portfd = -1;		/* serial port file descriptor */
long qid = -1;			/* IPC queue identifier */
long pid = -1;			/* our pid for IPC */
char ***configfile = NULL;	/* line-word-char array from rcfile */

int main(int argc, char *argv[]) {
  int max_clients = 0;		/* maximum number of RECEIVE clients */
  int ipcperms = 0;		/* IPC queue permissions */
  char *opt_port = NULL;	/* name of serial port */		
  char testbuf[10];
  struct x10msg send, rec, *ptr = NULL;

  /* see if we are already running */
  if (rdpidfile(PIDFILE, &pid, &qid) != -1) {
	fprintf(stderr, "%s already running as pid %li\n", argv[0], pid);
	exit(1);
  } 

  printf("twd x10 server (C)1996 Daniel R. Barrett\n");

  /* load and parse the config file, if one exists */
  configfile = load_rcfile(XEVENTDB);
  if (configfile != NULL) {
	int i;
	for(i=0; configfile[i] != NULL; i++) {
		if ( (samestring(configfile[i][0], "maxclients")) &&
		     (configfile[i][1] != NULL) ) {
			max_clients = strtol(configfile[i][1], NULL, 10);
		}
		if ( (samestring(configfile[i][0], "perm")) &&
		     (configfile[i][1] != NULL) ) {
			ipcperms = strtol(configfile[i][1], NULL, 8);
		}
		if ( (samestring(configfile[i][0], "port")) &&
		     (configfile[i][1] != NULL) ) {
			opt_port = configfile[i][1];
		}
	}
  }

  /* clean up any undefined variables */
  if (opt_port == NULL)
	opt_port = DEFAULTPORT;
  if (ipcperms == 0)
	ipcperms = OURIPCPERMS;

  /* become a daemon */
  switch(fork()) {
	case -1:		/* error */
		die("Unable to fork");
	case 0:			/* child continues */
		(void)setsid();
		break;
	default:		/* parent exits */
		exit(0);
  }

  /* install signal handlers for nice graceful exit */
  signal(SIGTERM, exit_handler);
  signal(SIGINT, exit_handler);
  signal(SIGQUIT, exit_handler);
  signal(SIGHUP, exit_handler);

  if (init_clients(max_clients) != 0)
	die("init_clients() memory error");

  pid = (long)getpid();

  qid = create_queue(ipcperms);
  if (qid == -1)
	die("can't open an IPC queue");

  /* write pid and IPC message queue qid to PIDFILE */
  if (mkpidfile(PIDFILE, pid, qid) == -1)
	die("can't create pidfile");

  /* open/initialise the RS232 port */
  portfd = open_port(opt_port);
  if (portfd == -1) {
	fprintf(stderr, "\nUnable to open port ");
        perror(opt_port);
	cleanup();
	exit(1);
  }

  /* test connection and reset dim register */
  sleep(1);
  (void) gets_fd(portfd, testbuf, 10);		/* dump any garbage */
  if (	(puts_fd(portfd, "R01") != 0) ||
  	(gets_fd(portfd, testbuf, 10) != 1) ||
	(strncmp(testbuf, "R01", 3) != 0)  ) {
		die("Unable to initialize the Two-Way");
  }


  /* main loop */
  while(1) {
	switch(get_input(pid, qid, &send)) {
		case 0:			/* no message */
			ptr = NULL;
			break;
		case 1:			/* send contains valid data */
			ptr = &send;
			break;
		default:		/* error */
			die("get_input() error");
	}
	switch(dialog(portfd, ptr, &rec)) {
		case 0:	
			break;
		case 1:			
			if (statfile(XEVENTDB) == TRUE) {
				free3array(configfile);
				configfile = load_rcfile(XEVENTDB);
			}
			if (configfile != NULL)
				(void)x10func(&rec, configfile);
			(void)tell_clients(qid, &rec);
			break;
		default:
			die("dialog() error");
	}
  }		
  exit(1);	/* never happens */
}


void exit_handler(int signum) {
/* signal handler for nice exit if killed */
  fprintf(stderr, "twd exiting on signal %i\n", signum);
  cleanup();
  exit(0);
}



void cleanup(void) {
/* Put all our toys away, pull up our panties and go home. */
  if (qid != -1)		/* zap the IPC queue */
	delete_queue(qid);
  if (portfd != -1)
	close_port(portfd);	/* restore tty params */
  unlink(PIDFILE);		/* zap the pid file */
  if (configfile != NULL)	/* free config file mem */
	free3array(configfile);
  return;
}



void die(char *reason) {
	cleanup();
	fprintf(stderr, "die(): %s: %s\n", reason, strerror(errno));
	exit(1);
}
