#define	_POSIX_SOURCE	1
/*
 * Serial I/O module for Two-Way interface
 * dannyb@bga.com	10/27/96
*/
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <termios.h>

#include "twoway.h"

#define TRUE    	1
#define FALSE   	0
#define TWBUFLEN	8	/* tw sends messages of 4-5 bytes. */
#define CHARDELAY	100000	/* delay between chars when writing */
#define COLLISIONRETRIES 5

/* global variables */
struct termios old_termios;


int open_port(char *portname) {
  int fd;
  fd = open(portname, O_RDWR | O_NOCTTY);
  if (fd == -1)
	return fd;
  if (set_tty(fd) != 0) {
	close(fd);
	return -1;
  }	
  return fd;
}

void close_port(int fd) {
  restore_tty(fd);
  close(fd);
  return;
}


int dialog(int portfd, struct x10msg *m, struct x10msg *r) {
/*
 * If m is NULL, just read port and return result to *r.
 * returns:	-1: read() or write() error (fatal)
		0:  no data available(r)
		1   data is valid(r)
*/
  char command[19][4];	/* r value, 16 units, function */
  int unitmask;
  int i;
  int k;
  int args;
  char reply[TWBUFLEN];
  int x, y;

  if (m == NULL) {		/* just read the port and return */
	x = gets_fd(portfd, reply, TWBUFLEN);
	if (x == 1)
		return stox10msg(reply, r);
	return x;
  }

  m->func[0] = toupper(m->func[0]);	/* convert function to uppercase */
  m->func[1] = toupper(m->func[1]);

  /* extract command string from the x10msg structure, count args. */
  args=0;
  /* Preset dim/bright register value if command is br or di */
  if ( (strcmp(m->func, "BR")==0) || (strcmp(m->func, "DI")==0) ) {
	if ( (m->brdi > 0) && (m->brdi < 21) )
		sprintf(command[args++], "R%02i", m->brdi);
  }
  /* append the module codes */
  for(i=1; i<17; i++) {
	unitmask = 1 << (i-1);
	if (m->modules & unitmask)
		sprintf(command[args++], "%c%02i", m->house, i);
  }
  /* append the function code */
  sprintf(command[args++], "%c%2s", m->house, m->func);

  for(i=0; i<COLLISIONRETRIES; i++) {
	for(k=0; k<args; k++) {
	  	if (puts_fd(portfd, command[k]) != 0)
			return -1;
		/* multiple br/di take DAYS. sleep to avoid NULL */ 
		if ( (strcmp(&command[k][1], "BR")==0) || 
			(strcmp(&command[k][1], "DI")==0) )
				sleep(5);
		x = gets_fd(portfd, reply, TWBUFLEN);
		if (x == -1)	/* fatal read error */
			return -1;
		if (x == 0)	/* no data; retransmit */
			break;
		if (strcmp(reply, "XXX") == 0)	/* collision */
			break;
		if (strcmp(reply, "???") == 0)	/* bad command */
			return 0;
		y = stox10msg(reply, r);
		if (k == args-1)
			return y;
	}
  }
  return 0;			/* Give up, return collision code. */
}



int stox10msg(char *s, struct x10msg *result) {
/*
 * Manage array of primes for all 16 house codes and call x10func()
 * for each function(as it affects primed modules).
 * returns:	0=no data, 1=valid data
*/
  int i;
  char h;
  int u = -1;
  char function[3];
  int houseint, unitint;        /* house = 0-15, unit = 0-15 */
  static int primes[16];
  static int resetprimes[16];
  static int initialized = FALSE;

  if (initialized == FALSE) {   /* start-up initialization */
        initialized = TRUE;
        for (i=0; i<15; i++) {
                primes[i] = 0;          /* clear all primes */
                resetprimes[i] = 0;     /* clear all resets */
        }
  }

  if (s == NULL)
	return 0;

  /* don't process a COLLISION or an ERROR response from twd */
  if ( (strcmp(s, "???") == 0) || (strcmp(s, "XXX") == 0) )
        return 0;
/*
 * If this is called with a module priming code, mark that module
 * as primed and return NULL.
*/
  if (sscanf(s, "%c%2d", &h, &u) == 2) {
        houseint = toupper(h)-65;
        unitint = u-1;
        if ( (houseint<0) || (houseint>15) || (unitint<0) || (unitint>15) )
                return 0;
        if (resetprimes[houseint]) {    /* reset (unit followed by funct) */
                resetprimes[houseint] = FALSE;
                primes[houseint] = 0;
        }
        /* mark unit as primed */
        primes[houseint] |= (1 << unitint);
        return 0;
  }
/*
 * If it is a function code, figure out which modules it affects and
 * return an x10 structure containing the house, modules, and function.
*/
  if (sscanf(s, "%c%2s", &h, function) == 2) {
        houseint = toupper(h)-65;
        if ( (houseint<0) || (houseint>15) )
                return 0;
        if (strcmp(function, "A0") == 0) {
                primes[houseint] = 0;           /* reset now */
        } else {
                resetprimes[houseint] = TRUE;   /* reset on next UNIT */
        }
        result->house = houseint + 65;
        result->modules = primes[houseint];
        strcpy(result->func, function);
        return 1;
  }
  return 0;
}



int gets_fd(int portfd, char *buf, int maxlen) {
/*
 * Read a line from portfd and return. Portfd is set for 1 second blocking.
 * Returns "???" on error so all returns will be 3 chars long.
*/
  int result;
  char *bufptr = buf;
  while(1) {
	result = read(portfd, bufptr, 1);
	if (result != 1)
		return result;	/* -1 = error, 0 = no data */
	if ( (bufptr - buf) >= (maxlen - 1) )
		continue;	/* prevents buffer overrun */
	/* snarfs all non-alphanumeric chars except '?' */
	if ( (isalnum(*bufptr)) || (*bufptr == '?') ) {
		bufptr++;
		continue;
	}
	if (*bufptr == '\n') {	/* got a complete line */
		*bufptr = '\0';
		bufptr = buf;
		if (strchr(buf, '?') != NULL)
			buf = "???";
		return 1;	/* 1 = "valid data" */
	}
  }
}



int puts_fd(int portfd, char *buf) {
/*
 * Write a 3-char string to fd, terminate it with <CR><LF>.
 * global macros:	CHARDELAY
*/
  unsigned int i, j;
  char line[6];

  strncpy(line, buf, 3);
  line[3] = '\r';
  line[4] = '\n';
  line[5] = '\0';
  
  for (i=0; i<strlen(line); i++) {
	if (write(portfd, &line[i], 1) != 1)
		return -1;
	for (j=0; j<CHARDELAY; j++);	/* delay between characters */
  }
  return 0;
}



int restore_tty(int fd) {
/* Restore port to original settings.
 * global variables:	old_termios.
*/
  return (tcsetattr(fd, TCSANOW, &old_termios));
}



int set_tty(int fd) {
/*
 * Set the serial port to 8 bits, 1 stop, no parity, and 1 second blocking.
 * with no special input or output processing.
*/
  struct termios t;
  if (tcdrain(fd) <0)           /* wait for pending output to be flushed */
        return -1;
  if (tcgetattr(fd, &t) < 0)    /* get current settings */
        return -1;
  (void)memcpy(&old_termios, &t, sizeof(struct termios));

  /* Input processing */
  t.c_iflag &= ~(       /* clear bits */
        BRKINT |                /* ignore break         */
        IGNBRK |                /* read break as '\0'   */
        IGNPAR |                /* ignore parity        */
        PARMRK |
        INPCK |
        ISTRIP |                /* don't strip 8 to 7   */
        INLCR |                 /* don't convert nl/cr  */
        IGNCR |
        ICRNL |
        IXON |                  /* ignore xon/xoff      */
        IXOFF
  );
  /* Output processing */
  t.c_oflag &= ~(OPOST);        /* no output processing */
  /* Hardware control settings */
  t.c_cflag &= (        /* set bits */
        CLOCAL |                /* use modem status lines       */
        CREAD |                 /* enable receiving     */
        CSIZE | CS8 |           /* 8 bits               */
        HUPCL                   /* hangup on last close */
  );            
  t.c_cflag &= ~(       /* clear bits */
        PARENB |                /* no parity            */
	CSTOPB			/* one stop bit 	*/
  );
  /* local mode flags */
  t.c_lflag &= (        /* set bits */
        NOFLSH
  );
  t.c_lflag &= ~(       /* clear bits */
        ECHO |                  /* no echoing 		*/
        ECHOE |
        ECHOK |
        ECHONL |
        ICANON |                /* disable canon input processing       */
        ISIG |                  /* no signals from spec. chars.         */
        TOSTOP |                /* no signal SIGTTOU    */      
        IEXTEN                  /* no extended flags    */
  );
  /* control characters */
  t.c_cc[VMIN] = 0;		/* return after reading n chars. */
  t.c_cc[VTIME] = 5;		/* return after n/10 sec */

  /* speed settings */
  if (cfsetispeed(&t, B2400) == -1) return -1;
  if (cfsetospeed(&t, B2400) == -1) return -1;
  /* write in new settings */
  if (tcsetattr(fd, TCSANOW, &t) < 0) return -1;
  return 0; 
}
